-- This is stolen from AceGUI-3.0, especially AceGUIWidget-EditBox.lua
-- the reason I stole this is because I need a widget that can show spoell name suggestions

local AceGUI = LibStub("AceGUI-3.0")
do
	local Type = "SUF_Spell_EditBox"
	local Version = 1
	local totalSpellsLoaded, spellLoader = 0
	local spells, indexedSpells = {}, {}
	local predictBackdrop = {
	  bgFile = "Interface\\ChatFrame\\ChatFrameBackground",
	  edgeFile = "Interface\\DialogFrame\\UI-DialogBox-Border",
	  edgeSize = 26,
	  insets = {left = 9, right = 9, top = 9, bottom = 9},
	}
	
	-- Spells have to gradually be loaded in to prevent the client from lagging, this starts as soon as the frame is shown
	-- as of 3.1 the spellID goes up to ~66,000 which means it'll take around 5 - 10 seconds for it to load them all
	-- it starts scanning as soon as one widget is shown given users have to actually move the mouse, type what they want etc
	-- it should result in them not noticing it does not have all the spell data yet
	local function startLoading()
		if( spellLoader ) then return end

		spellLoader = CreateFrame("Frame")
		spellLoader.timeElapsed = 0
		spellLoader.totalInvalid = 0
		spellLoader.index = 0
		spellLoader:SetScript("OnUpdate", function(self, elapsed)
			self.timeElapsed = self.timeElapsed + elapsed
			if( self.timeElapsed < 0.05 ) then return end
			self.timeElapsed = 0

			-- Too many invalid spells found will assume we found all there is that we can
			if( self.totalInvalid >= 5000 ) then
				self:Hide()
				return
			end

			-- Load as many spells in
			for i=spellLoader.index + 1, spellLoader.index + 500 do
				local name, _, icon = GetSpellInfo(i)
				
				-- The majority of spells that use the engineer gear icon are actually invalid spells that we can easily ignore
				-- since there are ~12000 not counting duplicate names that use this icon it's worthwhile to filter out these spells
				self.totalInvalid = self.totalInvalid + 1
				if( name and icon ~= "Interface\\Icons\\Trade_Engineering" ) then
					name = string.lower(name)
					
					if( not spells[name] ) then
						spells[string.lower(name)] = i
						table.insert(indexedSpells, name)
						
						totalSpellsLoaded = totalSpellsLoaded + 1
						self.totalInvalid = 0
					end
				end
			end
						
			-- Increment and do it all over!
			spellLoader.index = spellLoader.index + 500
		end)
	end

	-- Search for spells quickly
	local function searchSpells(self, query)
		for _, button in pairs(self.buttons) do button:Hide() end

		local usedButtons = 0
		for i=1, totalSpellsLoaded do
			local name = indexedSpells[i]
			if( string.match(name, query) ) then
				usedButtons = usedButtons + 1

				local spellName, _, spellIcon = GetSpellInfo(spells[name])
				local button = self.buttons[usedButtons]
				button.spellID = spells[name]
				button:SetFormattedText("|T%s:20:20:2:11|t %s", spellIcon, spellName)
				button:Show()

				-- Ran out of text to suggest :<
				if( usedButtons >= 8 ) then break end
			end
		end
		
		if( usedButtons > 0 ) then
			self:SetHeight(15 + usedButtons * 17)
			self:Show()
		else
			self:Hide()
		end
	end
			
	local function OnAcquire(self)
		self:SetHeight(26)
		self:SetWidth(200)
		self:SetDisabled(false)
		self:SetLabel()
		self.showbutton = true
	end
	
	local function OnRelease(self)
		self.frame:ClearAllPoints()
		self.frame:Hide()
		self:SetDisabled(false)
	end
			
	local function Control_OnEnter(this)
		this.obj:Fire("OnEnter")
	end
	
	local function Control_OnLeave(this)
		this.obj:Fire("OnLeave")
	end
	
	local function EditBox_OnEscapePressed(this)
		this:ClearFocus()
	end
	
	local function ShowButton(self)
		if( self.lasttext ~= "" ) then
			searchSpells(self.editbox.predictFrame, "^" .. string.lower(self.lasttext))
			self.editbox.predictFrame.selectedButton = nil
			self.editbox.predictFrame:Show()
		else
			self.editbox.predictFrame:Hide()
		end
			
		if self.showbutton then
			self.button:Show()
			self.editbox:SetTextInsets(0,20,3,3)
		end
	end
	
	local function HideButton(self)
		self.button:Hide()
		self.editbox:SetTextInsets(0,0,3,3)
		self.editbox.predictFrame:Hide()
	end
	
	local function EditBox_OnEnterPressed(this)
		if( this.predictFrame.selectedButton ) then
			this.predictFrame.buttons[this.predictFrame.selectedButton]:Click()
			this.predictFrame.selectedButton = nil
			return
		end
	
		local self = this.obj
		local value = this:GetText()
		local cancel = self:Fire("OnEnterPressed",value)
		if not cancel then
			HideButton(self)
		end
	end
	
	local function Button_OnClick(this)
		local editbox = this.obj.editbox
		editbox:ClearFocus()
		EditBox_OnEnterPressed(editbox)
	end
	
	local function EditBox_OnReceiveDrag(this)
		local self = this.obj
		local type, id, info = GetCursorInfo()
		if type == "spell" then
			local name = GetSpellName(id, info)
			self:SetText(name)
			self:Fire("OnEnterPressed" ,name)
			ClearCursor()
		end
		HideButton(self)
		AceGUI:ClearFocus()
	end
	
	local function EditBox_OnTextChanged(this)
		local self = this.obj
		local value = this:GetText()
		if value ~= self.lasttext then
			self:Fire("OnTextChanged",value)
			self.lasttext = value
			ShowButton(self)
		end
	end
		
	local function SetDisabled(self, disabled)
		self.disabled = disabled
		if disabled then
			self.editbox:EnableMouse(false)
			self.editbox:ClearFocus()
			self.editbox:SetTextColor(0.5,0.5,0.5)
			self.label:SetTextColor(0.5,0.5,0.5)
		else
			self.editbox:EnableMouse(true)
			self.editbox:SetTextColor(1,1,1)
			self.label:SetTextColor(1,.82,0)
		end
	end
	
	local function SetText(self, text, cursor)
		self.lasttext = text or ""
		self.editbox:SetText(text or "")
		self.editbox:SetCursorPosition(cursor or 0)
		HideButton(self)
	end
	
	local function SetLabel(self, text)
		if text and text ~= "" then
			self.label:SetText(text)
			self.label:Show()
			self.editbox:SetPoint("TOPLEFT",self.frame,"TOPLEFT",7,-18)
			self:SetHeight(44)
			self.alignoffset = 30
		else
			self.label:SetText("")
			self.label:Hide()
			self.editbox:SetPoint("TOPLEFT",self.frame,"TOPLEFT",7,0)
			self:SetHeight(26)
			self.alignoffset = 12
		end
	end
	
	local function Spell_OnClick(self)
		local name = GetSpellInfo(self.spellID)
		SetText(self.parent.parent, name, string.len(name))
	end
	
	local function Spell_OnEnter(self)
		self.parent.selectedButton = nil
		self:LockHighlight()
		
		GameTooltip:SetOwner(self, "ANCHOR_BOTTOMRIGHT")
		GameTooltip:SetHyperlink("spell:" .. self.spellID)
	end
	
	local function Spell_OnLeave(self)
		self:UnlockHighlight()
		GameTooltip:Hide()
	end

	local function Constructor()
		local num  = AceGUI:GetNextWidgetNum(Type)
		local frame = CreateFrame("Frame", nil, UIParent)
		local editbox = CreateFrame("EditBox", "AceGUI-3.0SUFSpellEditBox" .. num, frame, "InputBoxTemplate")

		-- Don't feel like looking up the specific callbacks for when a widget resizes, so going to be creative with SetPoint instead!
		local predictFrame = CreateFrame("Frame", nil, editbox)
		predictFrame:SetBackdrop(predictBackdrop)
		predictFrame:SetBackdropColor(0, 0, 0, 0.85)
		predictFrame:SetWidth(1)
		predictFrame:SetHeight(150)
		predictFrame:SetPoint("TOPLEFT", editbox, "BOTTOMLEFT", -6, 0)
		predictFrame:SetPoint("TOPRIGHT", editbox, "BOTTOMRIGHT", 0, 0)
		predictFrame:Hide()
		
		predictFrame.buttons = {}

		for i=1, 8 do
			local button = CreateFrame("Button", nil, predictFrame)
			button:SetHeight(17)
			button:SetWidth(1)
			button:SetPushedTextOffset(-2, 0)
			button:SetScript("OnClick", Spell_OnClick)
			button:SetScript("OnEnter", Spell_OnEnter)
			button:SetScript("OnLeave", Spell_OnLeave)
			button.parent = predictFrame
			button.editbox = editbox
			button:Hide()
			
			if( i > 1 ) then
				button:SetPoint("TOPLEFT", predictFrame.buttons[i - 1], "BOTTOMLEFT", 0, 0)
				button:SetPoint("TOPRIGHT", predictFrame.buttons[i - 1], "BOTTOMRIGHT", 0, 0)
			else
				button:SetPoint("TOPLEFT", predictFrame, 8, -8)
				button:SetPoint("TOPRIGHT", predictFrame, -7, 0)
			end

			-- Create the actual text
			local text = predictFrame:CreateFontString(nil, "ARTWORK", "GameFontNormal")
			text:SetHeight(1)
			text:SetWidth(1)
			text:SetJustifyH("LEFT")
			text:SetAllPoints(button)
			button:SetFontString(text)

			-- Setup the highlighting
			local texture = frame:CreateTexture(nil, "ARTWORK")
			texture:SetTexture("Interface\\QuestFrame\\UI-QuestTitleHighlight")
			texture:ClearAllPoints()
			texture:SetPoint("TOPLEFT", button, 0, -2)
			texture:SetPoint("BOTTOMRIGHT", button, 5, 2)
			texture:SetAlpha(0.70)

			button:SetHighlightTexture(texture)
			button:SetHighlightFontObject(GameFontHighlight)
			button:SetNormalFontObject(GameFontNormal)
			
			table.insert(predictFrame.buttons, button)
		end		
		
		local self = {}
		self.type = Type
		self.num = num

		self.OnRelease = OnRelease
		self.OnAcquire = OnAcquire

		self.SetDisabled = SetDisabled
		self.SetText = SetText
		self.SetLabel = SetLabel
		
		self.frame = frame
		frame.obj = self
		self.editbox = editbox
		editbox.obj = self
		editbox.predictFrame = predictFrame
		predictFrame.parent = self
		
		self.alignoffset = 30
		
		frame:SetHeight(44)
		frame:SetWidth(200)

		editbox:SetScript("OnShow", startLoading)
		editbox:SetScript("OnEnter", Control_OnEnter)
		editbox:SetScript("OnLeave", Control_OnLeave)
		
		editbox:SetAutoFocus(false)
		editbox:SetFontObject(ChatFontNormal)
		editbox:SetScript("OnEscapePressed", EditBox_OnEscapePressed)
		editbox:SetScript("OnEnterPressed", EditBox_OnEnterPressed)
		editbox:SetScript("OnTextChanged", EditBox_OnTextChanged)
		editbox:SetScript("OnReceiveDrag", EditBox_OnReceiveDrag)
		editbox:SetScript("OnMouseDown", EditBox_OnReceiveDrag)

		editbox:SetTextInsets(0,0,3,3)
		editbox:SetMaxLetters(256)
		
		editbox:SetPoint("BOTTOMLEFT", frame, "BOTTOMLEFT", 6, 0)
		editbox:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", 0, 0)
		editbox:SetHeight(19)
		
		local label = frame:CreateFontString(nil,"OVERLAY","GameFontNormalSmall")
		label:SetPoint("TOPLEFT", frame, "TOPLEFT", 0, -2)
		label:SetPoint("TOPRIGHT", frame, "TOPRIGHT", 0, -2)
		label:SetJustifyH("LEFT")
		label:SetHeight(18)
		self.label = label
		
		local button = CreateFrame("Button",nil,editbox,"UIPanelButtonTemplate")
		button:SetWidth(40)
		button:SetHeight(20)
		button:SetPoint("RIGHT", editbox, "RIGHT", -2, 0)
		button:SetText(OKAY)
		button:SetScript("OnClick", Button_OnClick)
		button:Hide()
		
		self.button = button
		button.obj = self

		AceGUI:RegisterAsWidget(self)
		return self
	end
	
	AceGUI:RegisterWidgetType(Type, Constructor, Version)
end
